@@ -2,11 +2,15 @@
 {-# OPTIONS_HADDOCK prune #-}
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleInstances #-}
+#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 708
+-- For the IsList instance:
+{-# LANGUAGE TypeFamilies #-}
+#endif
 
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.DList
--- Copyright   :  (c) Don Stewart 2006-2009, (c) Sean Leather 2013
+-- Copyright   :  (c) 2006-2009 Don Stewart, 2013-2014 Sean Leather
 -- License     :  See LICENSE file
 --
 -- Maintainer  :  sean.leather@gmail.com
@@ -47,19 +51,31 @@ import Prelude hiding (concat, foldr, map, head, tail, replicate)
 import qualified Data.List as List
 import Control.DeepSeq (NFData(..))
 import Control.Monad as M
-import Data.Monoid
 import Data.Function (on)
 import Data.String (IsString(..))
 
-import Data.Foldable (Foldable)
 import qualified Data.Foldable as F
 
-#ifdef __GLASGOW_HASKELL__
-import Text.Read (Lexeme(Ident), lexP, parens, prec, readPrec, readListPrec,
-                  readListPrecDefault)
+#if !MIN_VERSION_base(4,8,0)
+import Data.Monoid
+import Data.Foldable (Foldable)
+import Control.Applicative(Applicative(..))
 #endif
 
-import Control.Applicative(Applicative(..), Alternative, (<|>))
+#ifdef __GLASGOW_HASKELL__
+
+import Text.Read (Lexeme(Ident), lexP, parens, prec, readPrec, readListPrec,
+                  readListPrecDefault)
+
+#if __GLASGOW_HASKELL__ >= 708
+import GHC.Exts (IsList)
+-- This is for the IsList methods, which conflict with fromList, toList:
+import qualified GHC.Exts
+#endif
+
+#endif
+
+import Control.Applicative(Alternative, (<|>))
 import qualified Control.Applicative (empty)
 
 -- | A difference list is a function that, given a list, returns the original
@@ -271,3 +287,12 @@ instance IsString (DList Char) where
   fromString = fromList
   {-# INLINE fromString #-}
 
+#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 708
+instance IsList (DList a) where
+  type Item (DList a) = a
+  fromList = fromList
+  {-# INLINE fromList #-}
+  toList = toList
+  {-# INLINE toList #-}
+#endif
+
